/*
  pg.cpp

  File designed to be linked with non Windows application to enable
  run it under Windows.
  TEdit class is used to emulate stdout.
  Did't want to use EasyWin because it doesn't run under Win32.

  Date     By  Change
  -------- --- -------------------------------------------------------------
  01/21/94 PG  initial version
*/

#include <stdio.h>
#include <stdarg.h>
#include <io.h>
#include <sys\stat.h>
#include <fcntl.h>
#include <owl\owlpch.h>
#include <owl\applicat.h>
#include <owl\edit.h>
#include <owl\inputdia.h>

void pump_messages();
int printf(const char *fmt, ...);

int Argc;
char **Argv;

//_______________________________PGEdit_____________________________________

class TPGEdit : public TEdit {
public:
    TPGEdit(TWindow *parent) : TEdit(parent, 0, 0, 0, 0, 0, 0, 0, TRUE) {}
    void print(char *);
};

void
TPGEdit::print(char *buf){
    static UINT start=0, end=0;

    if (GetNumLines() >= 200){
        SetRedraw(FALSE);
        DeleteLine(0);
        SetRedraw(TRUE);
    }
    SetSelection(end, end);
    Insert(buf);
    GetSelection(start, end);
}

TPGEdit *Winp;

//_______________________________PGApp______________________________________

class TPGApp : public TApplication {
public:
    void InitMainWindow();
    TPGApp() : TApplication() {}
    int MessageLoop();
    void test_break();
};

void
TPGApp::test_break(){
    if(BreakMessageLoop) throw 0;
}

void
TPGApp::InitMainWindow(){
    Winp = new TPGEdit(GetMainWindow());
    SetMainWindow(new TFrameWindow(0, "PG", Winp));
}

// borrowed from \bc4\source\applicat.cpp
//
// general message loop: retrieves and processes messages from the OWL
// application's message queue until BreakMessageLoop becomes true
//
int
TPGApp::MessageLoop(){
    MessageLoopResult = 0;
  do{
    try {
        pump_messages();
// call your real application here
        for(int i=0; i<1000; i++){
            printf(">>>>>>>>>>>>>>>>>>line #%d<<<<<<<<<<<<<<<<<<<\r\n", i);
        }
        break;
    }
    catch (TXOwl& x) {
      MessageLoopResult = x.Unhandled(this, IDS_OKTORESUME);
      if (MessageLoopResult != 0) {
        ::PostQuitMessage(MessageLoopResult);
        break;
      }
    }
    catch (xmsg& x) {
      MessageLoopResult = Error(x, 0, IDS_OKTORESUME);
      if (MessageLoopResult != 0) {
        ::PostQuitMessage(MessageLoopResult);
        break;
      }
    }
    catch (Bad_cast& x) {
      MessageLoopResult = Error(xmsg(string(typeid(x).name())), 0, IDS_OKTORESUME);
      if (MessageLoopResult != 0) {
        ::PostQuitMessage(MessageLoopResult);
        break;
      }
    }
    catch (Bad_typeid& x) {
      MessageLoopResult = Error(xmsg(string(typeid(x).name())), 0, IDS_OKTORESUME);
      if (MessageLoopResult != 0) {
        ::PostQuitMessage(MessageLoopResult);
        break;
      }
    }
    catch (const int i) { break; }
  } while(0);
  BreakMessageLoop = FALSE;
  return MessageLoopResult;
}

TPGApp *PGApp;

//______________________________functions___________________________________

void
pump_messages(){
    PGApp->PumpWaitingMessages();     // pumps any waiting messages
    PGApp->test_break();
}

int
printf(const char *fmt, ...){
    va_list ap;
    char buf[256];
    int r;

    va_start(ap, fmt);
    r = vsprintf(buf, fmt, ap);
    if(*buf != '\0') Winp->print(buf);
    pump_messages();
    va_end(ap);
    return r;
}

char *
p_gets(const char *prompt, char *buf, const int size){
    pump_messages();
    *buf = '\0';
    TInputDialog(Winp, "", prompt, buf, size>80?80:size).Execute();
    return buf;
}

int
OwlMain(int argc, char *argv[]){
    Argc = argc;
    Argv = argv;
    PGApp = new TPGApp();
    PGApp->nCmdShow = SW_SHOWMAXIMIZED;
    return PGApp->Run();
}
